#!/bin/sh
# Copyright (C) 2006 OpenWrt.org
# Copyright (C) 2006 Fokus Fraunhofer <carsten.tittel@fokus.fraunhofer.de>

alias debug=${DEBUG:-:}

# newline
N="
"

_C=0
NO_EXPORT=1

hotplug_dev() {
	env -i ACTION=$1 INTERFACE=$2 /sbin/hotplug-call net
}

append() {
	local var="$1"
	local value="$2"
	local sep="${3:- }"
	
	eval "export ${NO_EXPORT:+-n} -- \"$var=\${$var:+\${$var}\${value:+\$sep}}\$value\""
}

reset_cb() {
	config_cb() { return 0; }
	option_cb() { return 0; }
}
reset_cb

config () {
	local cfgtype="$1"
	local name="$2"
	
	export ${NO_EXPORT:+-n} CONFIG_NUM_SECTIONS=$(($CONFIG_NUM_SECTIONS + 1))
	name="${name:-cfg$CONFIG_NUM_SECTIONS}"
	append CONFIG_SECTIONS "$name"
	[ -n "$NO_CALLBACK" ] || config_cb "$cfgtype" "$name"
	export ${NO_EXPORT:+-n} CONFIG_SECTION="$name"
	export ${NO_EXPORT:+-n} "CONFIG_${CONFIG_SECTION}_TYPE=$cfgtype"
}

option () {
	local varname="$1"; shift
	local value="$*"
	export ${NO_EXPORT:+-n} "CONFIG_${CONFIG_SECTION}_${varname}=$value"
	[ -n "$NO_CALLBACK" ] || option_cb "$varname" "$*"
}

config_rename() {
	local OLD="$1"
	local NEW="$2"
	local oldvar
	local newvar

	[ "$OLD" -a "$NEW" ] || return
	for oldvar in `set | grep ^CONFIG_${OLD}_ | \
		sed -e 's/\(.*\)=.*$/\1/'` ; do
		newvar="CONFIG_${NEW}_${oldvar##CONFIG_${OLD}_}"
		eval "export ${NO_EXPORT:+-n} \"$newvar=\${$oldvar}\""
		unset "$oldvar"
	done
	export ${NO_EXPORT:+-n} CONFIG_SECTIONS="$(echo " $CONFIG_SECTIONS " | sed -e "s, $OLD , $NEW ,")"

	[ "$CONFIG_SECTION" = "$OLD" ] && export ${NO_EXPORT:+-n} CONFIG_SECTION="$NEW"
}

config_unset() {
	config_set "$1" "$2" ""
}

config_clear() {
	local SECTION="$1"
	local oldvar

	export ${NO_EXPORT:+-n} CONFIG_SECTIONS="$(echo " $CONFIG_SECTIONS " | sed -e "s, $OLD , ,")"
	export ${NO_EXPORT:+-n} CONFIG_SECTIONS="${SECTION:+$CONFIG_SECTIONS}"

	for oldvar in `set | grep ^CONFIG_${SECTION:+${SECTION}_} | \
		sed -e 's/\(.*\)=.*$/\1/'` ; do 
		unset $oldvar 
	done
}

config_load() {
	local file="$UCI_ROOT/tmp/etc/config/$1"
	_C=0
	export ${NO_EXPORT:+-n} CONFIG_SECTIONS=
	export ${NO_EXPORT:+-n} CONFIG_NUM_SECTIONS=0
	export ${NO_EXPORT:+-n} CONFIG_SECTION=
	[ -e "$file" ] && {
		. $file
	} || return 1

	${CONFIG_SECTION:+config_cb}
}

config_get() {
	case "$3" in
		"") eval "echo \"\${CONFIG_${1}_${2}}\"";;
		*)  eval "export ${NO_EXPORT:+-n} -- \"$1=\${CONFIG_${2}_${3}}\"";;
	esac
}

# config_get_bool <variable> <section> <option> [<default>]
config_get_bool() {
	local _tmp
	config_get "_tmp" "$2" "$3"
	case "$_tmp" in
		1|on|enabled) export ${NO_EXPORT:+-n} "$1=1";;
		0|off|disabled) export ${NO_EXPORT:+-n} "$1=0";;
		*) eval "$1=${4:-0}";;
	esac
}

config_set() {
	local section="$1"
	local option="$2"
	local value="$3"
	local old_section="$CONFIG_SECTION"
	CONFIG_SECTION="$section"
	option "$option" "$value"
	CONFIG_SECTION="$old_section"
}

config_foreach() {
	local function="$1"
	local type="$2"
	local section cfgtype
	[ -z "$CONFIG_SECTIONS" ] && return 0
	for section in ${CONFIG_SECTIONS}; do
		config_get cfgtype "$section" TYPE
		[ -n "$type" -a "$cfgtype" != "$type" ] && continue
		eval "$function \"\$section\""
	done
}

load_modules() {
	[ -d /tmp/etc/modules.d ] && {
		cd /tmp/etc/modules.d
		sed 's/^[^#]/insmod &/' $* | ash 2>&- || :
	}
}

include() {
	local file

	for file in $(ls $1/*.sh 2>/dev/null); do
		. $file
	done
}

find_mtd_part() {
	local PART="$(grep "\"$1\"" /proc/mtd | awk -F: '{print $1}')"
	local PREFIX=/dev/mtdblock

	PART="${PART##mtd}"
	[ -d /dev/mtdblock ] && PREFIX=/dev/mtdblock/
	echo "${PART:+$PREFIX$PART}"
}

strtok() { # <string> { <variable> [<separator>] ... }
	local tmp
	local val="$1"
	local count=0

	shift

	while [ $# -gt 1 ]; do
		tmp="${val%%$2*}"

		[ "$tmp" = "$val" ] && break

		val="${val#$tmp$2}"

		export ${NO_EXPORT:+-n} "$1=$tmp"; count=$((count+1))
		shift 2
	done

	if [ $# -gt 0 -a "$val" ]; then
		export ${NO_EXPORT:+-n} "$1=$val"; count=$((count+1))
	fi

	return $count
}


jffs2_mark_erase() {
	local part="$(find_mtd_part "$1")"
	[ -z "$part" ] && {
		echo Partition not found.
		return 1
	}
	echo -e "\xde\xad\xc0\xde" | mtd -qq write - "$1"
}

get_month() {

    case "$1" in
        "january") echo "1" ;;
        "february") echo "2" ;;
        "march") echo "3" ;;
        "april") echo "4" ;;
        "may") echo "5" ;;
        "june") echo "6" ;;
        "july") echo "7" ;;
        "august") echo "8" ;;
        "september") echo "9" ;;
        "october") echo "10" ;;
        "november") echo "11" ;;
        "december") echo "12" ;;
        *) echo "0" ;;
    esac
}

get_week() {

    case "$1" in
        "first") echo "1" ;;
        "second") echo "2" ;;
        "third") echo "3" ;;
        "fourth") echo "4" ;;
        "last") echo "5" ;;
        *) echo "0" ;;
    esac
}

get_day() {

    case "$1" in
        "monday") echo "1" ;;
        "tuesday") echo "2" ;;
        "wednesday") echo "3" ;;
        "thursday") echo "4" ;;
        "friday") echo "5" ;;
        "saturday") echo "6" ;;
        *) echo "0" ;;
    esac
}

get_offset() {
    s=0
    h=0
    m=0
    j=""
    offh=""
    offm=""
    i="`echo $1  | cut -d "," -f1`"
    if [ "$(expr index "$i" -)" != "0" ]; then
        s=1
    fi
    if [ "$(expr index "$i" :)" != "0" ]; then
        m=`echo $i  | cut -d ":" -f2`
        h=`echo "$i" | grep -Eo '[0-9]+'`
    else
        h=`echo "$i" | grep -Eo '[0-9]+'`
    fi

    if [ $s -eq 1 ]; then
      h=$(($h * -1))
    fi

    if [ $2 -eq 60 ]; then
        echo ""
    elif [ $s -eq 1 ]; then
        offh="$h"
        offm=$(($m+$2))
        if [ "$offm" -gt 60 ]; then
            offh=$(($offh - $(($offm/60))))
            offm=$(($offm%60))
        fi
        if [ $offm -eq 0 ]; then
            echo "$offh"
        elif [ $offm -lt 10 ]; then
            echo "$offh:0$offm"
        else
            echo "$offh:$offm"
        fi
    else
        offh="$h"
        offm=$(($m-$2));
        if [ $offm -lt 0 ]; then
            offh=$(($offh-1))
            th=$(($offm / 60))
            th=$(($th * -1))
            offh=$(($offh - $th))
            offm=$((offm % 60 + 60))
            if [ $offm -eq 60 ];then
                offm=0
                offh=$(($offh+1))
            fi
            if [ $offh -lt 0 ];then
                offh=$(($offh+1))
                offm=$((60 - $offm))
                if [ $offh -eq 0 ];then
                    $offh="-$offh"
                fi
            fi
        fi
        if [ $offm -eq 0 ]; then
            echo "$offh"
        elif [ $offm -lt 10 ]; then
            echo "$offh:0$offm"
        else
            echo "$offh:$offm"
        fi
    fi

}

get_timezone() {

config_load timezone

config_get location "timezone" zoneinfo
config_get dst_start "timezone" dst_start
config_get dst_end "timezone" dst_end
config_get dst_offset "timezone" dst_offset
config_get dst_enable "timezone" dst_enable

month="`echo $dst_start | cut -d "." -f1`"
week="`echo $dst_start | cut -d "." -f2`"
day="`echo $dst_start | cut -d "." -f3 | sed 's/\/.*//'`"

month1="`get_month $month`"
week1="`get_week $week`"
day1="`get_day $day`"
if [ "$(expr index "$dst_start" \/)" == "0" ]; then
    time1="2:00"
else
    time1="`echo $dst_start | sed 's/.*\///'`"
fi

month="`echo $dst_end | cut -d "." -f1`"
week="`echo $dst_end | cut -d "." -f2`"
day="`echo $dst_end | cut -d "." -f3 | sed 's/\/.*//'`"

month2="`get_month $month`"
week2="`get_week $week`"
day2="`get_day $day`"
if [ "$(expr index "$dst_end" \/)" == "0" ]; then
    time2="2:00"
else
    time2="`echo $dst_end | sed 's/.*\///'`"
fi
if [ -z "$dst_offset" ]; then
    dst_offset="60"
fi

zone="`echo $location | cut -d " " -f1`"
city="`echo $location | cut -d "(" -f2 | cut -d ")" -f1`"
str="`cat /usr/lib/webif/timezones.csv | grep "$zone" | grep "$city"`"

zone=`echo "$zone" | sed 's/\//+/g'`
city=`echo "$city" | sed 's/\//+/g'`
str=`echo "$str" | sed 's/\//+/g'`

localtime="`echo "$str" | sed s/"$zone"//g`"
localtime="`echo "$localtime" | sed s/.*"$city"//g`"
localtime="`echo $localtime | cut -d " " -f1`"
localtime="`echo $localtime | sed 's/+/\//g`"

if [ "$dst_enable" == "1" ]; then
    if [ -n "$dst_start" ] && [ -n "$dst_end" ]; then
        i=""
        if [ "$(expr index "$localtime" ,)" == "0" ]; then
            i="$localtime"
        else
            t=""
            i="`echo $localtime  | cut -d "," -f1`"
            if [ "$(expr index "$i" :)" != "0" ]; then
                m="`echo $i  | cut -d ":" -f2`"
                t="`echo "$m" | grep -Eo '[0-9]+'`"
            else
                t="`echo "$i" | grep -Eo '[0-9]+'`"
            fi
            j="`echo "$i" | sed s/$t.*/$t/`"
            i="$j"
        fi
        j=""
        if [ "$dst_offset" != "60" ]; then
            j="$i"DST""`get_offset $i $dst_offset`",M$month1.$week1.$day1/$time1,M$month2.$week2.$day2/$time2"
        else
            j="$i"DST",M$month1.$week1.$day1/$time1,M$month2.$week2.$day2/$time2"
        fi
        echo "$j"

    elif [ "$(expr index "$localtime" ,)" != "0" ]; then
        echo "$localtime"
    elif [ -z "$dst_start" ] || [ -z "$dst_end" ]; then
        i="$localtime"
        j=""
        if [ "$dst_offset" != "60" ]; then
            j="$i"DST""`get_offset $i $dst_offset`""
        else
            j="$i"DST""
        fi
        echo "$j"
    else
        i="$localtime"
        j=""
        if [ "$dst_offset" != "60" ]; then
            j="$i"DST""`get_offset $i $dst_offset`",M$month1.$week1.$day1/$time1,M$month2.$week2.$day2/$time2"
        else
            j="$i"DST",M$month1.$week1.$day1/$time1,M$month2.$week2.$day2/$time2"
        fi
        echo "$j"
    fi
else
    if [ "$(expr index "$localtime" ,)" == "0" ]; then
        echo "$localtime"
    else
        t=""
        i="`echo $localtime  | cut -d "," -f1`"
        if [ "$(expr index "$i" :)" != "0" ]; then
            m="`echo $i  | cut -d ":" -f2`"
            t="`echo "$m" | grep -Eo '[0-9]+'`"
        else
            t="`echo "$i" | grep -Eo '[0-9]+'`"
        fi
        j="`echo "$i" | sed s/$t.*/$t/`"
        echo "$j"
    fi
fi
}
